@@ -49,9 +49,14 @@ class IOBase(ReprMixin, ABC):
     def name(self, value):
         self._name = value
 
-    @property
-    def _default_name(self):
-        return ""
+    _default_name = ""
+
+    def has_default_name(self):
+        """
+
+        :return: True if this object is using its default name, false otherwise
+        """
+        return not self._name
 
 
 def get_name(x: Union[str, int, IOBase]):
@@ -62,7 +67,7 @@ def get_name(x: Union[str, int, IOBase]):
     return x.name
 
 
-class Descriptor(ReprMixin, ABC):
+class Descriptor(IOBase, ABC):
     """Base class for descriptors.
 
     Descriptors describe a set of possible values for a single input.
@@ -70,8 +75,8 @@ class Descriptor(ReprMixin, ABC):
     the possible input space.
     """
 
-    def __init__(self, **kwargs):
-        super().__init__(**kwargs)
+    def __init__(self, name=""):
+        super().__init__(name=name)
         self.platypus_type = NotImplemented
 
     @abstractmethod
@@ -88,6 +93,8 @@ class Descriptor(ReprMixin, ABC):
         """Takes a value in the range 0-1 and returns a valid value for this parameter"""
         pass
 
+    _default_name = "input"
+
 
 class AnyValue(Descriptor):
     """A :class:`Descriptor` that can take on any possible value.
