@@ -16,16 +16,19 @@ from pathlib import Path
 from deprecated.sphinx import deprecated
 
 # BESOS Imports
-import eppy_funcs as ef
-import config
-import objectives
-from errors import ModeError
-from besostypes import PathLike
+from besos import eppy_funcs as ef
+from besos import config
+from besos import objectives
+from besos.errors import ModeError, InstallationError
+from besos.besostypes import PathLike
 
 
 # TODO: Make this a method of the building class when we add it
 def get_idf_version(building):
-    """ get energyplus version from idf or json file """
+    """Get energyplus version from idf or json building
+    :param building: IDF object to find version of.
+    :returns: String of version (ex. '9.3.0')
+    """
     mode = ef.get_mode(building)
     if mode == "idf":
         return building.idfobjects["VERSION"][0].Version_Identifier
@@ -33,18 +36,72 @@ def get_idf_version(building):
         try:
             return building["Version"]["Version 1"]["version_identifier"]
         except KeyError:
-            warnings.warn(f"Cannot find IDF version, substituting default.")
-            return config.energy_plus_version
+            try:
+                return building["epJSON_schema_version"]
+            except:
+                raise ModeError(
+                    mode="json",
+                    message=f"Cannot find IDF version, check epJSON formatting.",
+                )
     else:
         raise ModeError(mode)
 
 
+def get_idf_version_pth(path):
+    """Get energyplus version from idf or epJSON path
+    :param path: Path to idf to epJSON file.
+    :returns: String of version (ex. '9.3.0')
+    """
+    path = str(path)
+    version_lines = (
+        ""  # string containing the important lines of the json file to later
+    )
+    version = ""
+    flag = False  # used to save the lines following 'Version' to version_lines
+    if path.endswith(".idf"):
+        with open(path, "r") as read_obj:  # opening idf file to read
+            # Read all lines in the file one by one
+            for line in read_obj:
+                if flag:
+                    version_lines += line.rstrip(
+                        "\n"
+                    )  # adds the line after 'Version,' is seen in case of line break
+                    break
+                # For each line, check if line contains 'Version,'
+                if ("Version," in line or "VERSION," in line) and not line.startswith(
+                    "!"
+                ):
+                    version_lines += line.rstrip("\n")
+                    flag = True
+        version = (
+            version_lines.split(";")[0].split(",")[-1].replace(" ", "").replace("=", "")
+        )
+    elif path.endswith(".epJSON"):
+        readobj = open(path, "r")
+        jsonText = readobj.read()
+        readobj.close()
+        jsonObj = json.loads(jsonText)
+        try:
+            version = jsonObj["Version"]["Version 1"]["version_identifier"]
+        except KeyError:
+            try:
+                version = jsonObj["epJSON_schema_version"]
+            except KeyError:
+                message = f"Version object not found within epJSON file, consider updating epJSON file\nepJSON File: {path}"
+                raise ModeError("json", message=message)
+    return version
+
+
 @deprecated(
     version="1.6.0",
     reason=("Version is derived automatically, this check is redundant."),
 )
 def check_idf_version(building, version):
-    """ check if the version of energyplus matches the version in idf/json. """
+    """check if the version of energyplus matches the version in idf/json.
+    :param building: Building to check
+    :param version: EnergyPlus version
+    :returns: None.
+    """
     idf_version = get_idf_version(building).replace("-", ".")
     version = version.replace("-", ".")
     if idf_version != version[:3]:
@@ -71,9 +128,13 @@ def has_hvac_templates(building) -> bool:
 
 def run_building(building, out_dir=config.out_dir, version=None, **eplus_args):
     """Run energy plus on a building object and return results
-
-    If out_dir is not defined, the results will not be saved
-    in the file system"""
+    :param building_path: Path to building file
+    :param out_dir: Path to store EnergyPlus output files, if out_dir is not defined the results will not be saved
+    :param version: Version of building file
+    :param ep_path: Path to EnergyPlus if installed in unexpected directory
+    :param expand_objects: Boolean if '--expandobjects' option to be appended to command
+    :returns: A dictionary of EnergyPlus outputs
+    """
     # backwards compatibility
     if version:
         warnings.warn(
@@ -111,13 +172,24 @@ def run_energyplus(
     err_dir: PathLike = config.err_dir,
     schema_file=None,
     error_mode="Silent",
-    version=config.energy_plus_version,
+    version=None,
     ep_path=None,
     expand_objects: bool = False,
 ):
-    """Run energy plus.
-    This method is intended to work as similar to the cli tool as possible"""
-
+    """Run EnergyPlus. This method is intended to work as similar to the cli tool as possible
+    :param building_path: Path to building file
+    :param out_dir: Path to store EnergyPlus output files
+    :param epw: Path to epw file
+    :param err_dir: Path to store EnergyPlus error files
+    :param schema_file: Path to data dictionary for EnergyPlus
+    :param error_mode: Error mode selection
+    :param version: Version of building file
+    :param ep_path: Path to EnergyPlus if installed in unexpected directory
+    :param expand_objects: Boolean if '--expandobjects' option to be appended
+    :returns: None
+    """
+    if version is None:
+        version = get_idf_version_pth(building_path)
     ep_exe_path, ep_directory = get_ep_path(version, ep_path)
     schema_file = schema_file or Path(ep_directory, "Energy+.idd")
 
@@ -155,34 +227,58 @@ def run_energyplus(
         raise e
 
 
-def get_ep_path(version, ep_path=None):
-    """ get energyplus installation path by version"""
+def get_ep_path(idf_version, ep_path=None):
+    """get EnergyPlus installation path by idf_version
+    :param idf_version: idf_version of EnergyPlus
+    :param ep_path: Optional - path to EnergyPlus if installed in different location then expected
+    :returns: (path to EnergyPlus executable, path to EnergyPlus)
+    """
+    if len(idf_version) == 3:
+        if idf_version == "9.0":
+            idf_version = "9-0-1"
+        else:
+            idf_version = idf_version.replace(".", "-") + "-0"
+    else:
+        idf_version = idf_version.replace(".", "-")
     if ep_path is not None:
-        ep_directory = ep_path
+        ep_directory = os.path.abspath(ep_path)
+        if not os.path.exists(ep_directory):
+            message = f"'ep_path' does not exist.\n'ep_path': {ep_path}"
+            raise InstallationError(message=message)
+        try:
+            cmd = [os.path.join(ep_directory, "energyplus"), "--version"]
+            out, err = subprocess.Popen(cmd, stdout=subprocess.PIPE).communicate()
+            ep_path_version = (
+                str(out).split("Version ")[-1].split("-")[0].replace(".", "-")
+            )
+        except:
+            message = f"'ep_path' provided does not contain an energyplus file\n'ep_path': {ep_path}"
+            raise InstallationError(message=message)
+        if ep_path_version != idf_version:
+            message = f"'ep_path' provided is a different version of EnergyPlus then 'idf_version' argument\n'ep_path': {ep_path}\n'idf_version': {idf_version}"
+            raise InstallationError(message=message)
         if platform.system() == "Windows":
             ep_exe = os.path.join(ep_directory, "energyplus.exe")
-        elif platform.system() == "Linux":
-            ep_exe = os.path.join(ep_directory, "energyplus")
         else:
             ep_exe = os.path.join(ep_directory, "energyplus")
     else:
-        if len(version) == 3:
-            if version == "9.0":
-                version = "9-0-1"
-            else:
-                version = version.replace(".", "-") + "-0"
-        else:
-            version = version.replace(".", "-")
         # this is duplicated from eppy.runner.run_functions.paths_from_version
         if platform.system() == "Windows":
-            ep_directory = "C:/EnergyPlusV{version}".format(version=version)
+            ep_directory = "C:/EnergyPlusV{idf_version}".format(idf_version=idf_version)
             ep_exe = os.path.join(ep_directory, "energyplus.exe")
         elif platform.system() == "Linux":
-            ep_directory = "/usr/local/EnergyPlus-{version}".format(version=version)
+            ep_directory = "/usr/local/EnergyPlus-{idf_version}".format(
+                idf_version=idf_version
+            )
             ep_exe = os.path.join(ep_directory, "energyplus")
         else:
-            ep_directory = "/Applications/EnergyPlus-{version}".format(version=version)
+            ep_directory = "/Applications/EnergyPlus-{idf_version}".format(
+                idf_version=idf_version
+            )
             ep_exe = os.path.join(ep_directory, "energyplus")
+
+    if not os.path.exists(ep_directory) or not os.path.exists(ep_exe):
+        raise InstallationError(version=idf_version)
     return ep_exe, ep_directory
 
 
