@@ -24,10 +24,10 @@ from eppy.bunch_subclass import BadEPFieldError
 from shapely.geometry import LineString, Point, Polygon
 
 # BESOS Imports
-import config
-import eplus_funcs as eplus
-from IDF_class import IDF, view_epJSON
-from errors import ModeError
+from besos import config
+from besos import eplus_funcs as eplus
+from besos.IDF_class import IDF, view_epJSON
+from besos.errors import ModeError
 
 
 def view_building(building):
@@ -95,12 +95,13 @@ def get_mode(building) -> str:
 def get_idf(
     idf_file: str = config.files.get("idf"),
     idd_file: str = None,
-    version=config.energy_plus_version,
+    version=None,
     output_directory=config.out_dir,
     ep_path=None,
 ) -> IDF:
     """Uses eppy to read an idf file and generate the corresponding idf object"""
-
+    if version is None:
+        version = eplus.get_idf_version_pth(idf_file)
     if idd_file is None:
         # set iddname to default value if needed.
         _, ep_directory = eplus.get_ep_path(version, ep_path)
@@ -130,12 +131,13 @@ def get_building(
     data_dict=None,
     output_directory=config.out_dir,
     mode=None,
-    version=config.energy_plus_version,
+    version=None,
     ep_path=None,
 ):
     """Get building from directory"""
     if mode is None and building is None:
         building = config.files.get("building")
+        version = eplus.get_idf_version_pth(building)
         mode = config.energy_plus_mode
     if mode is None:
         ending = Path(building).suffix
@@ -149,9 +151,15 @@ def get_building(
             )
     if building is None:
         building = config.files[mode]
+    if version is None:
+        version = eplus.get_idf_version_pth(building)
     if mode == "idf":
         idf = get_idf(
-            idf_file=building, idd_file=data_dict, output_directory=output_directory
+            idf_file=building,
+            version=version,
+            idd_file=data_dict,
+            output_directory=output_directory,
+            ep_path=ep_path,
         )
         return idf
     elif mode == "json":
