@@ -13,13 +13,11 @@ The Evaluators wrap their respective modeling tools with the evaluator interface
 import copy
 import json
 import os
-import tempfile
 import time
 import warnings
 from abc import ABC, abstractmethod
-from functools import lru_cache, wraps
 from pathlib import Path
-from typing import Callable, Optional, Sequence, Tuple, Union
+from typing import Callable, Optional, Tuple, Union, Iterable, Sequence
 from warnings import warn
 
 # External Libraries
@@ -34,39 +32,40 @@ from pyehub.outputter import output_excel
 from tqdm.auto import tqdm
 
 # BESOS Imports
-import config
-import eplus_funcs as eplus
-import eppy_funcs as ef
-import pyehub_funcs as pf
-import sampling
-from problem import Problem
-from besostypes import PathLike
+from besos import config
+from besos import eplus_funcs as eplus
+from besos import eppy_funcs as ef
+from besos import pyehub_funcs as pf
+from besos import sampling
+from besos.problem import Problem
+from besos.besostypes import PathLike
 
 
-def _list_cache(
-    enabled: bool = config.cache["enabled"],
-    size: Optional[int] = config.cache["size"],
-    typed: bool = config.cache["typed"],
-):
-    """Decorator that allows the lru_cache (last recently used cache)
-    to work on functions that receive lists"""
+def _freeze(value):
+    """This function converts values to hashable representations, so that
+    they can be used as inputs for caching.
 
-    # TODO add complete type hint
-    def _list_cache_decorator(f: Callable) -> Callable:
-        if enabled:
-            cache_f = lru_cache(size, typed)(f)
+    :param value: a value to attempt to convert to a hashable equivalent
+    :return: the value, or a tuple with the same contents.
+    """
+    # We may want to add other types to this function
+    # set -> frozenset, or saving the type of the object
+    # in addition to the values.
+    # currently these are not needed.
 
-            @wraps(cache_f)
-            def safe_f(individual, *args, **kwargs):
-                return cache_f(tuple(individual), *args, **kwargs)
-
-            safe_f.cache_clear = cache_f.cache_clear
-            return safe_f
-        # add a no-op as the cache clear operation so it does not error if disabled in the config
-        f.cache_clear = lambda: None
-        return f
-
-    return _list_cache_decorator
+    # note that this try/except is not the same as
+    # isinstance(value, Hashable), which
+    # incorrectly detects pd.Series as hashable
+    try:
+        hash(value)
+        return value
+    except TypeError:
+        pass
+    if isinstance(value, dict):
+        return frozenset((_freeze(k), _freeze(v)) for k, v in value.items())
+    if isinstance(value, Iterable):
+        return tuple(_freeze(x) for x in value)
+    raise TypeError(f"unhashable type: '{type(value)}'")
 
 
 class AbstractEvaluator(ABC):
@@ -87,7 +86,6 @@ class AbstractEvaluator(ABC):
         problem: Problem,
         error_mode: str = "Failfast",
         error_value: tuple = None,
-        multi: bool = False,
         progress_bar: bool = True,
     ):
         """
@@ -100,17 +98,13 @@ class AbstractEvaluator(ABC):
         :param error_value: The value of the evaluation if an error occurs.
             Incompatible with error_mode='Failfast'. Must be a tuple consisting of
             values for the objectives followed by values for the constraints.
-        :param multi: whether or not to use multiprocessing, using this disables cache
         :param progress_bar: whether or not to display a progress bar
         """
         self.problem = problem
         self.error_mode = error_mode
         self.error_value = self._convert_error_mode_from_old_format(error_value)
         self.validate_error_mode()
-        self.multi = multi
-        # this makes the cache instance-specific instead of class-specific
-        if not self.multi:
-            self._call = _list_cache()(self._uncached_call)
+        self._cache = {}
         self.progress_bar = progress_bar
         self.pbar = None
 
@@ -169,27 +163,6 @@ class AbstractEvaluator(ABC):
         """
         pass
 
-    def _uncached_call(self, values: Sequence, **kwargs) -> tuple:
-        # This method is a workaround to allow lru_cache to be instance-specific and work on __call__
-        try:
-            self.validate(values)
-            if self.problem.depend_params:
-                values = self.problem.overwrite_values(values)
-            result = self.eval_single(values, **kwargs)
-        except Exception as e:
-            if self.error_mode != "Silent":
-                msg = ""
-                if self.problem.inputs is not None:
-                    msg += f'for inputs: {self.problem.names("inputs")}'
-                msg += f"problematic values were: {values}"
-                warn(msg)
-            if self.error_mode == "Failfast":
-                raise e
-            else:
-                # TODO: Move error_value to flat format as well
-                result = self.error_value
-        return result
-
     def __call__(self, values: Sequence, **kwargs) -> tuple:
         """Returns the objective results for a single list of parameter values.
 
@@ -202,16 +175,31 @@ class AbstractEvaluator(ABC):
         # Override eval_single, not this method.
         # Values can be empty to allow evaluating at the current state, but this is not the default behaviour
         # If this is not supported, the validate function of a subclass should reject an empty list
-        if self.multi:
-            return self._uncached_call(values, **kwargs)
-        else:
-            return self._call(values, **kwargs)
+
+        key = _freeze(values), _freeze(kwargs)
+        if key in self._cache:
+            return self._cache[key]
+        try:
+            self.validate(values)
+            result = self.eval_single(values, **kwargs)
+        except Exception as e:
+            if self.error_mode != "Silent":
+                msg = ""
+                if self.problem.num_inputs > 0:
+                    msg += f'for inputs: {self.problem.names("inputs")} '
+                msg += f"problematic values were: {values}"
+                warn(msg)
+            if self.error_mode == "Failfast":
+                raise e
+            else:
+                result = self.error_value
+        self._cache[key] = result
+        return result
 
     def cache_clear(self) -> None:
         """Clears any cached vales of calls to this evaluator.
         This should be called whenever the evaluator's outputs could have changed."""
-        if not self.multi:
-            self._call.cache_clear()
+        self._cache = {}
 
     def validate(self, values: Sequence) -> None:
         """Takes a list of values and checks that they are a valid input for this evaluator."""
@@ -220,22 +208,22 @@ class AbstractEvaluator(ABC):
                 f"Wrong number of input values."
                 f"{len(values)} provided, {self.problem.num_inputs} expected"
             )
-        for p, value in zip(self.problem.inputs, values):
-            if not p.validate(value):
-                raise ValueError(f"Invalid value {value} for parameter {p}")
+        for descriptor, value in zip(self.problem.value_descriptors, values):
+            if not descriptor.validate(value):
+                raise ValueError(f"Invalid value {value} for descriptor {descriptor}")
 
-    def estimate_time(self, df: DF, processes: int = 4) -> None:
+    def estimate_time(self, df: DF, processes: int = 1) -> None:
         """Prints out a very rough estimate of the amount of time a job will take to complete.
         Will underestimate smaller sample sets but becomes more accurate as they become larger.
 
         :param df: a DataFrame where each row represents valid input values for this Evaluator.
-        :param processes: amount of cores to use (only relevant when using multiprocessing)
+        :param processes: amount of cores to use
         """
         start = time.time()
         res = self(df.iloc[0])
         end = time.time()
 
-        if self.multi and processes > 1:
+        if processes > 1:
             estimate = (end - start) * df.shape[0] / processes
         else:
             estimate = (end - start) * df.shape[0]
@@ -244,19 +232,30 @@ class AbstractEvaluator(ABC):
         s = np.floor((estimate % 3600) % 60)
         print(f"Estimated time for completion: {h} hours, {m} minutes, {s} seconds")
 
-    def df_apply(self, df: DF, keep_input=False, processes: int = 4, **kwargs) -> DF:
+    def df_apply(self, df: DF, keep_input=False, processes: int = 1, **kwargs) -> DF:
         """Applies this evaluator to an entire dataFrame, row by row.
 
         :param df: a DataFrame where each row represents valid input values for this Evaluator.
         :param keep_input: whether to include the input data in the returned DataFrame
-        :param processes: amount of cores to use (only relevant when using multiprocessing)
+        :param processes: amount of cores to use
         :return: Returns a DataFrame with one column containing the results for each objective.
         """
-        if self.multi and processes > 1:
+        if processes > 1:
             ddf = from_pandas(df, npartitions=processes)
-            expected_output_shape = pd.DataFrame(
-                columns=list(range(self.problem.num_outputs))
-            )
+            # TODO: Use information from problem to give accurate types for this shape.
+            try:
+                if kwargs["keep_dirs"]:
+                    expected_output_shape = pd.DataFrame(
+                        columns=list(range(self.problem.num_outputs + 1))
+                    )
+                else:
+                    expected_output_shape = pd.DataFrame(
+                        columns=list(range(self.problem.num_outputs))
+                    )
+            except:
+                expected_output_shape = pd.DataFrame(
+                    columns=list(range(self.problem.num_outputs))
+                )
             dask_result = ddf.apply(
                 self,
                 axis=1,
@@ -422,14 +421,12 @@ class EvaluatorGeneric(AbstractEvaluator):
         problem: Problem,
         error_mode: str = "Failfast",
         error_value: Sequence = None,
-        multi: bool = False,
         progress_bar: bool = True,
     ):
         """
         :param evaluation_func: a function that takes as input an list of values,
                 and gives as output a tuple of the objective values for that point in the solution space
         :param problem: description of the inputs and outputs the evaluator will use
-        :param multi: whether or not to use multiprocessing, using this disables cache
         :param progress_bar: whether or not to display a progress bar
         """
 
@@ -437,7 +434,6 @@ class EvaluatorGeneric(AbstractEvaluator):
             problem=problem,
             error_mode=error_mode,
             error_value=error_value,
-            multi=multi,
             progress_bar=progress_bar,
         )
         self._evaluation_func = evaluation_func
@@ -505,14 +501,12 @@ class AdaptiveSR(AbstractEvaluator, ABC):
         reference: AbstractEvaluator = None,
         error_mode: str = "Failfast",
         error_value: Sequence = None,
-        multi: bool = False,
     ):
         self.reference: AbstractEvaluator = reference
         super().__init__(
             problem=reference.problem,
             error_mode=error_mode,
             error_value=error_value,
-            multi=multi,
         )
 
         self.model = None
@@ -628,8 +622,6 @@ class EvaluatorEP(AbstractEvaluator, ABC):
         err_dir: PathLike = config.err_dir,
         error_mode: str = "Failfast",
         error_value: Optional[Sequence] = None,
-        multi: bool = False,
-        distributed: bool = False,
         version=None,
         progress_bar: bool = True,
         ep_path: PathLike = None,
@@ -648,9 +640,6 @@ class EvaluatorEP(AbstractEvaluator, ABC):
             Silent: Evaluation will return the `error_value` for any lists of values that raise an error.
             Print: Same as silent, but warnings are printed to stderr for any errors.
         :param error_value: The value of the evaluation if an error occurs. Incompatible with error_mode='Failfast'.
-        :param multi: whether or not to use multiprocessing, using this disables cache
-        :param distributed: Using this flag will save eplus configuration files into memory.
-            It lowers performance, but it enables parallel computing with remote workers
         :param version: Deprecated
         :param progress_bar: whether or not to display a progress bar
         :param epw_file: Deprecated. Use epw instead. Path to the epw file representing the weather.
@@ -660,7 +649,6 @@ class EvaluatorEP(AbstractEvaluator, ABC):
             problem=problem,
             error_mode=error_mode,
             error_value=error_value,
-            multi=multi,
             progress_bar=progress_bar,
         )
         if out_dir is None:
@@ -697,24 +685,15 @@ class EvaluatorEP(AbstractEvaluator, ABC):
             assert version == eplus.get_idf_version(building), "Incorrect version"
 
         self.epw = epw
-        self.distributed = distributed
 
-        if self.multi:
-            self.err_dir = None
-            self.out_dir = None
-
-        if self.multi and self.distributed:
-            # save the epw to memory. When evaluate is run on a worker,
-            # the epw will be moved with the object
-            with open(self.epw, "r") as f:
-                self.epw_ = f.read()
-                f.close()
+        # epw can be read when needed, instead of here
+        self.epw_contents = None
 
     def df_apply(
         self,
         df: DF,
         keep_input=False,
-        processes: int = 2,
+        processes: int = 1,
         keep_dirs: bool = False,
         *,
         out_dir=config.out_dir,
@@ -723,89 +702,48 @@ class EvaluatorEP(AbstractEvaluator, ABC):
 
         :param df: a DataFrame where each row represents valid input values for this Evaluator.
         :param keep_input: whether to include the input data in the returned DataFrame
-        :param processes: amount of cores to use (only relevant when using multiprocessing)
+        :param processes: amount of cores to use
         :param keep_dirs: whether or not keep output directory
         :return: Returns a DataFrame with one column containing the results for each objective.
         """
+        temp_self = self.out_dir
+        temp_out = out_dir
+        if not keep_dirs:
+            out_dir = self.out_dir = None
+
         result = super().df_apply(
             df,
-            keep_input=False,
+            keep_input=keep_input,
             processes=processes,
             keep_dirs=keep_dirs,
             out_dir=out_dir,
         )
-        # Fixes Dataframe output when using Multiply or Add mode
-        for j, parameter in enumerate(self.problem.inputs):
-            if not isinstance(parameter, int):
-                # FIXME: This code block looks broken
-                if parameter.mode != "Set":
-                    names = self.problem.names("inputs")
-                    if parameter.mode != "Multiply":
-                        df.rename(
-                            columns={names[j]: names[j] + " Multiplier"},
-                            inplace=True,
-                            errors="raise",
-                        )
-                    if parameter.mode != "Add":
-                        df.rename(
-                            columns={names[j]: names[j] + " Addend"},
-                            inplace=True,
-                            errors="raise",
-                        )
-        if keep_input:
-            result = pd.concat([df, result], axis=1)
         # TODO: Rework automatic column naming
         if keep_dirs:
             result.rename(
                 columns={result.columns[-1]: "output_dir"}, inplace=True, errors="raise"
             )
+        if not keep_dirs:
+            self.out_dir = temp_self
+            out_dir = temp_out
+
         self.cleanup_pbar()
         return result
 
-    def save_epw_to_temp(self):
-        """Used when evaluate is called and distributed is True
-        Writes epw to a temp dir
-        Remember to call temp_dir.cleanup() after using the dir"""
-        temp_dir = tempfile.TemporaryDirectory()
-        temp_epw = Path(temp_dir.name, "temp_epw.epw")
-
-        with open(temp_epw, "w") as f:
-            f.write(self.epw_)
-            f.close()
-
-        return temp_epw, temp_dir
-
-    def eval_single(self, values: list, out_dir=None, keep_dirs=False):
+    def eval_single(self, values: Sequence, out_dir=None, keep_dirs=False):
         out_dir = out_dir or self.out_dir
         err_dir = self.err_dir
         if keep_dirs:
             err_dir = out_dir = ef.generate_dir(out_dir)
-        current_building = copy.deepcopy(self.building)
-        for p, value in zip(self.problem.inputs, values):
-            # apply the modification of each parameter to the building using the corresponding value
-            p.transformation_function(current_building, value)
-        if self.multi and self.distributed:
-            # the remote worker can't access the original epw file
-            # Here, the worker writes the epw from memory to a file
-            temp_epw, temp_dir = self.save_epw_to_temp()
-            results = eplus.run_building(
-                current_building,
-                out_dir=out_dir,
-                err_dir=err_dir,
-                epw=temp_epw,
-                error_mode=self.error_mode,
-                ep_path=self.ep_path,
-            )
-            temp_dir.cleanup()
-        else:
-            results = eplus.run_building(
-                current_building,
-                out_dir=out_dir,
-                err_dir=err_dir,
-                epw=self.epw,
-                error_mode=self.error_mode,
-                ep_path=self.ep_path,
-            )
+        current_building = self._generate_building_from_row(values)
+        results = eplus.run_building(
+            current_building,
+            out_dir=out_dir,
+            err_dir=err_dir,
+            epw=self.epw,
+            error_mode=self.error_mode,
+            ep_path=self.ep_path,
+        )
         outputs = tuple((objective(results) for objective in self.problem.outputs))
         constraints = tuple(
             (constraint(results) for constraint in self.problem.constraints)
@@ -851,6 +789,7 @@ class EvaluatorEP(AbstractEvaluator, ABC):
         self.cache_clear()
         self._epw = value
 
+    # REMOVE?
     def generate_building(self, df: DF, index: int, file_name: str) -> None:
         """generate idf file
 
@@ -859,16 +798,31 @@ class EvaluatorEP(AbstractEvaluator, ABC):
         :param file_name: file name used to save as.
         :return: None
         """
-        l = df.values.tolist()[index][: len(self.problem.inputs)]
-        current_building = copy.deepcopy(self.building)
-        for p, value in zip(self.problem.inputs, l):
-            p.transformation_function(current_building, value)
+        l = df.values.tolist()[index][: len(self.problem.value_descriptors)]
+        current_building = self._generate_building_from_row(l)
         if isinstance(current_building, dict):
             with open(f"{file_name}.epJSON", "w") as fp:
                 json.dump(current_building, fp)
         else:
             current_building.saveas(f"{file_name}.idf")
 
+    # This "apply all of the parameters" behaviour is duplicated in at least 2 other places.
+    # EvaluatorEH.eval_single and pyehub_funcs.pyehub_parameter_editor
+    def _generate_building_from_row(self, row):
+        """Generates a copy of this evaluator's building from a row of data"""
+        self.validate(row)
+        current_building = copy.deepcopy(self.building)
+        values = {
+            descriptor: value
+            for descriptor, value in zip(self.problem.value_descriptors, row)
+        }
+        # We could verify if row has column names,
+        # and check that those match, warning if there is a mismatch.
+        # This could also be done in validate()
+        for parameter in self.problem.parameters:
+            parameter.transformation_function(current_building, values)
+        return current_building
+
 
 class EvaluatorEH(AbstractEvaluator, ABC):
     """This evaluator uses a Problem to modify an energy hub, and then solve it."""
@@ -881,7 +835,6 @@ class EvaluatorEH(AbstractEvaluator, ABC):
         err_dir: PathLike = config.err_dir,
         error_mode: str = "Failfast",
         error_value: Optional[Sequence] = None,
-        multi: bool = False,
         progress_bar: bool = True,
     ):
         """
@@ -895,7 +848,6 @@ class EvaluatorEH(AbstractEvaluator, ABC):
             Silent: Evaluation will return the `error_value` for any lists of values that raise an error.
             Print: Same as silent, but warnings are printed to stderr for any errors.
         :param error_value: The value of the evaluation if an error occurs. Incompatible with error_mode='Failfast'.
-        :param multi: whether or not to use multiprocessing, using this disables cache
         :param progress_bar: whether or not to display a progress bar
         """
 
@@ -903,7 +855,6 @@ class EvaluatorEH(AbstractEvaluator, ABC):
             problem=problem,
             error_mode=error_mode,
             error_value=error_value,
-            multi=multi,
             progress_bar=progress_bar,
         )
         # TODO: Either fully port the evaluator over to the config.py or switch everything over to config.yaml
@@ -914,22 +865,25 @@ class EvaluatorEH(AbstractEvaluator, ABC):
         with open(Path(config.baseDir, "config.yaml"), "r") as file:
             self.config_settings = yaml.safe_load(file)
 
-    # Need to overwrite __call__ due to default __call__ caching failing on time series values.
-    def __call__(self, values: Sequence, **kwargs) -> tuple:
-        # TODO: add caching that works with time series values
-        return self._uncached_call(values, **kwargs)
-
     # override of validate due to pyehub model inputs being lists
-    def validate(self, values: list) -> None:
+    def validate(self, values: Sequence) -> None:
         if len(values) != self.problem.num_inputs:
             raise ValueError(
                 f"Wrong number of input values."
                 f"{len(values)} provided, {self.problem.num_inputs} expected"
             )
 
-    def eval_single(self, values: list) -> tuple:
+    def eval_single(self, values: Sequence) -> tuple:
         current_hub = copy.deepcopy(self.hub)
-        pf.pyehub_parameter_editor(current_hub, self.problem.inputs, values)
+
+        # this is duplicated in EvaluatorEP._generate_building_from_row
+        self.validate(values)
+        value_pairs = {
+            descriptor: value
+            for descriptor, value in zip(self.problem.value_descriptors, values)
+        }
+        for parameter in self.problem.parameters:
+            parameter.transformation_function(current_hub, value_pairs)
 
         objectives = self.problem.names("outputs")
         current_hub.compile()
