@@ -27,11 +27,18 @@ import pandas as pd
 from packaging.version import Version
 
 # BESOS Imports
-import config
-import eppy_funcs as ef
-from IO_Objects import Objective
-from errors import ModeError
-from besostypes import PathLike
+from besos import config
+from besos import eppy_funcs as ef
+from besos.IO_Objects import Objective
+from besos.errors import ModeError
+from besos.besostypes import PathLike
+
+import logging
+from logging import debug as lgd
+import logging
+
+log = logging.getLogger(__name__)
+log.addHandler(logging.NullHandler())
 
 
 def _get_data_dict_regex():
@@ -471,13 +478,13 @@ class VariableReader(EPReader):
         Energyplus docs:
         https://bigladdersoftware.com/epx/docs/8-9/input-output-reference/input-for-output.html#outputvariable
 
-        :param key_value: What to measure (e.g. Electricity:Facility)
-        :param variable_name:  The kind of variable to use (e.g. Output:Variable)
-        :param class_name: the type of variable to read from.
-        :param frequency: the reporting frequency of the variable.
-        :param func: a function used to aggregate the results read from the file.
-        :param name: the name of the output column corresponding to this objective
+        :param key_value: The designation or location where to read the variable (e.g. Zone name, or Wall name, or * for all matches with variable_name available).
+        :param variable_name: The variable name you want to measure (e.g. "Site Outdoor Air Drybulb").
+        :param frequency: The reporting frequency of the variable.
+        :param func: A function used to aggregate the results read from the file.
+        :param name: The name of the output column corresponding to this objective.
         """
+
         super().__init__(
             class_name=class_name, frequency=frequency, func=func, name=name
         )
@@ -488,7 +495,9 @@ class VariableReader(EPReader):
             self._add_repr("variable_name")
 
     def results_name(self):
-        return f"{self.key_value},{self.variable_name}"
+        # E+ forces key_values to upper case in the eso file
+        # To ensure matching, the input key_value is forced to uppercase.
+        return f"{self.key_value.upper()},{self.variable_name}"
 
     @property
     def _default_name(self):
@@ -501,7 +510,7 @@ class VariableReader(EPReader):
 
         if self.frequency:
             meter_results = results[(results_name, self.frequency)]
-            return meter_results
+            return self._process(meter_results)
 
         for (name, _), v in results.items():
             if self.key_value != "*":
