@@ -7,18 +7,20 @@ These parameters are separate from the value that they take on during any evalua
 # Python Core Libraries
 import warnings
 from abc import ABC, abstractmethod
-from typing import Callable, Dict, Iterable, List, Tuple
+from typing import Callable, Dict, Iterable, List, Tuple, Union
 
 # External Libraries
 import platypus
 from deprecated.sphinx import deprecated
 
 # BESOS Imports
-import eppy_funcs as ef
-import pyehub_funcs as pf
-from IO_Objects import AnyValue, Descriptor, DummySelector, IOBase, Selector
-from config import range_parameter as conf
-from errors import ModeError
+from besos import eppy_funcs as ef
+from besos import pyehub_funcs as pf
+from besos import besos_utils
+from besos.IO_Objects import AnyValue, Descriptor, DummySelector, Selector
+from besos.IO_Objects import ReprMixin
+from besos.config import range_parameter as conf
+from besos.errors import ModeError
 
 
 class RangeParameter(Descriptor):
@@ -33,14 +35,17 @@ class RangeParameter(Descriptor):
     rbf_type = "R"
 
     def __init__(
-        self, min_val: float = conf.get("min"), max_val: float = conf.get("max")
+        self,
+        min_val: float = conf.get("min"),
+        max_val: float = conf.get("max"),
+        name="",
     ):
         """
 
         :param max_val: the minimum value
         :param min_val: the maximum value
         """
-        super().__init__()
+        super().__init__(name=name)
         if min_val > max_val:
             raise ValueError("minimum is larger than maximum")
         self.min = min_val
@@ -70,60 +75,9 @@ class RangeParameter(Descriptor):
     def __str__(self):
         return f"{self.__class__.__name__} [{self.min}, {self.max}]"
 
-
-class DependentParameter(Descriptor):
-    """ A parameter that the value of it depends on a range parameter"""
-
-    def __init__(self, index: int, factor, mode):
-        """
-
-        :param index: the index of the target range parameter in parameter list
-        :param mode: mode of dependent parameter.
-            If mode = 'sum', value = factor - target_val.
-            If mode = 'subtract', value = target_val - factor.
-            If mode = 'multiple', value = factor * target_val.
-            If mode= 'divide', value = factor / target_val
-            If mode = 'power', value = target_val ** factor
-        :param factor: the factor for calculation
-        """
-        super().__init__()
-        mode_list = ["sum", "subtract", "multiple", "divide", "power"]
-        if mode not in mode_list:
-            raise ValueError(f"DependentParameter doesn't support mode {mode}")
-        self.mode = mode
-        self.factor = factor
-        self.index = index
-        self._add_reprs(["mode", "factor", "index"])
-        self.platypus_type = platypus.Real(0, self.factor)
-
-    def validate(self, value):
-        """Checks if value is a valid value for this parameter.
-
-        :param value:
-        """
-        return isinstance(value, (int, float))
-
-    def sample(self, value):
-        """Return a value based on mode and factor
-
-        :param value:
-        """
-        return self.get_value(value)
-
-    def get_value(self, value):
-        if self.mode == "sum":
-            return self.factor - value
-        elif self.mode == "subtract":
-            return value - self.factor
-        elif self.mode == "multiple":
-            return self.factor * value
-        elif self.mode == "divide":
-            return self.factor / value
-        else:
-            return value ** self.factor
-
-    def __str__(self):
-        return f"{self.__class__.__name__} index = {self.index}, mode = {self.mode}, factor = {self.factor}"
+    @property
+    def _default_name(self):
+        return str(self)
 
 
 class _Category(platypus.Subset):
@@ -164,6 +118,11 @@ class CategoryParameter(Descriptor):
     def sample(self, value):
         return self.options[int(len(self.options) * value)]
 
+    # This is not a great default
+    @property
+    def _default_name(self):
+        return f"Category{self.options}"
+
 
 class AbstractFieldSelector(Selector, ABC):
     """Base class for selectors that modify one field in one or more objects in an EnergyPlus building"""
@@ -407,14 +366,14 @@ class GenericSelector(Selector):
         for attr in ("set", "get", "setup"):
             self._add_repr(attr, f"_{attr}", True)
 
-    def set(self, building, value):
-        """A custom function for setting this field in the building to the provided value
+    def set(self, building, *values):
+        """Redirects to the custom function given. The building and values are passed to the function.
 
         :param building: the building to modify
-        :param value: the value to set this field to
+        :param values: the values to use when modifying the building
         """
         if self._set is not None:
-            return self._set(building, value)
+            return self._set(building, *values)
         raise NotImplementedError
 
     def get(self, building):
@@ -439,48 +398,75 @@ class GenericSelector(Selector):
 # Parameters
 
 
-class Parameter(IOBase):
-
-    mode_options = {"Set", "Multiply", "Add"}
-
+class Parameter(ReprMixin):
     def __init__(
         self,
         selector: Selector = None,
-        value_descriptor: Descriptor = None,
-        mode="Set",
+        value_descriptors: Union[Descriptor, List[Descriptor]] = None,
         name="",
+        *,
+        value_descriptor: Descriptor = None,
     ):
         """
 
         :param selector: a Selector describing how to modify the building
-        :param value_descriptor: a Descriptor specifying which values to use
-        :param mode: the desired transformation function to use
-            Set: set the sample as the value
-            Multiply: multiply the pre-existing value by the sample factor
-            Add: add the sample value onto the pre-existing value
+        :param value_descriptors: a Descriptor specifying which values to use
         :param name: the name of the Parameter. Used for readability
             and column labelling.
         """
-        super().__init__(name=name)
+        super().__init__()
         self.selector = selector or DummySelector()
-        self.value_descriptor = value_descriptor or AnyValue()
-        assert mode in self.mode_options, f"Invalid mode option"
-        self.mode = mode
-        self._add_reprs(["selector", "value_descriptor"], True)
 
-    def transformation_function(self, building, value) -> None:
+        # handle old call format
+        if value_descriptor is not None and value_descriptors is not None:
+            raise ValueError("Cannot use both value_descriptor and value_descriptors")
+        elif value_descriptor:
+            warnings.warn(
+                FutureWarning("Use value_descriptors instead of value_descriptor.")
+            )
+            value_descriptors = value_descriptor
+
+        # assign default value
+        if value_descriptors is None:
+            # no inputs is probably not useful, but an empty list allows it
+            value_descriptors = [AnyValue()]
+        self.value_descriptors = besos_utils.listify(value_descriptors)
+
+        # handle name as input to a parameter to support old call signature
+        if name:
+            descriptor = self.value_descriptor
+            if descriptor.has_default_name():
+                descriptor.name = name
+            else:
+                warnings.warn(
+                    UserWarning(
+                        f"This parameter's descriptor is already named "
+                        f"{descriptor.name}."
+                        f"The name used as an input ({name}) will be discarded."
+                    )
+                )
+
+        self._add_reprs(["selector", "value_descriptors"], True)
+
+    def transformation_function(self, building, value_dict) -> None:
         """Mutates the building based on the value provided."""
-        if self.mode == "Multiply":
-            self.selector.multiply(building, value)
-        if self.mode == "Add":
-            self.selector.add(building, value)
-        if self.mode == "Set":
-            self.selector.set(building, value)
+        relevant_values = (
+            value_dict[descriptor] for descriptor in self.value_descriptors
+        )
+        self.selector.set(building, *relevant_values)
 
+    @deprecated(
+        reason="Does not support multiple descriptors per parameter."
+        "Use the sample method of the this Parameter's Descriptor(s) instead."
+    )
     def sample(self, value: float):
         """Takes a value in the range 0-1 and returns a valid value for this parameter"""
         return self.value_descriptor.sample(value)
 
+    @deprecated(
+        reason="Does not support multiple Descriptors per Parameter."
+        "Use the validate method of this Parameter's Descriptor(s) instead."
+    )
     def validate(self, value):
         """Checks if value is a valid value for this parameter.
 
@@ -493,16 +479,51 @@ class Parameter(IOBase):
         self.selector.setup(building)
 
     @property
-    def _default_name(self):
+    @deprecated(
+        reason="Parameters are no longer nameable. "
+        "Use the _default_name(s) or name(s) "
+        "of this Parameter's Descriptor(s) instead"
+    )
+    def _default_name(self) -> str:
         """The name to use for this Parameter if no name was provided"""
         if hasattr(self.selector, "field_name"):
             return self.selector.field_name
+        else:
+            warnings.warn(UserWarning, f"No name found for parameter {repr(self)}")
+            return "Unnamed"
 
     @property
+    @deprecated(
+        reason="Does not support multiple Descriptors per Parameter."
+        "Use the platypus_type of this Parameter's Descriptor(s) instead."
+    )
     def platypus_type(self):
         """The platypus equivalent of this parameter"""
         return self.value_descriptor.platypus_type
 
+    @property
+    @deprecated(
+        reason="Does not support multiple Descriptors per Parameter."
+        "Use the value_descriptors of this Parameter instead."
+    )
+    def value_descriptor(self) -> Descriptor:
+        num = len(self.value_descriptors)
+        if num == 1:
+            return self.value_descriptors[0]
+        raise ValueError(
+            f"this parameter has {num} value descriptors."
+            "You are trying to use an outdated feature that"
+            "assumes there is exactly one descriptor."
+        )
+
+    @property
+    @deprecated(
+        reason="Parameters are no longer nameable. "
+        "Use the name(s) of this Parameter's Descriptor(s) instead"
+    )
+    def name(self) -> str:
+        return self.value_descriptor.name
+
 
 @deprecated(
     version="1.6.0",
@@ -514,8 +535,6 @@ class Parameter(IOBase):
 class ParameterEH:
     """A parameter for use with EnergyHub"""
 
-    mode_options = {"Set", "Multiply", "Add"}
-
     # this method overrides the default behaviour when trying to create a new ParameterEH
     # instead of creating a ParameterEH, it creates a PathSelector,
     # puts it in a Parameter, and returns that parameter.
@@ -523,21 +542,17 @@ class ParameterEH:
         cls,
         parameter_path: List[str] = None,
         value_descriptor: Descriptor = None,
-        mode: str = "Set",
         **kwargs,
     ):
         """Swaps out the ParameterEH for a Parameter and PathSelector for backwards compatibility.
 
         :param parameter_path: the path to the parameter to modify for the EnergyHub
         :param value_descriptor: a Descriptor specifying which values to use
-        :param mode: the desired transformation function to use
-          Set: set the sample as the value
-          Multiply: multiply the pre-existing value by the sample factor
-          Add: add the sample value onto the pre-existing value
+        :return a Parameter containing the appropriate PathSelector and descriptor.
         """
         selector = PathSelector(parameter_path)
         return Parameter(
-            selector=selector, value_descriptor=value_descriptor, mode=mode, **kwargs
+            selector=selector, value_descriptors=value_descriptor, **kwargs
         )
 
 
@@ -572,12 +587,16 @@ class wwrSelector(Selector):
         ef.one_window(building)
 
 
-def wwr(value_descriptor=RangeParameter(0.01, 0.99), **kwargs) -> Parameter:
+def wwr(value_descriptor=None, **kwargs) -> Parameter:
     """Makes a window-to-wall-ratio parameter.
 
     :param value_descriptor: a parameter describing the valid window-wall-ratios
+        Defaults to RangeParameter(0.01, 0.99, name="Window to Wall Ratio")
     :return: Parameter
     """
+    value_descriptor = value_descriptor or RangeParameter(
+        0.01, 0.99, name="Window to Wall Ratio"
+    )
     if isinstance(value_descriptor, RangeParameter):
         min_val, max_val = value_descriptor.min, value_descriptor.max
         if not (0 < min_val < max_val < 1):
@@ -594,10 +613,7 @@ def wwr(value_descriptor=RangeParameter(0.01, 0.99), **kwargs) -> Parameter:
         )
 
     selector = wwrSelector()
-    name = kwargs.pop("name", "Window to Wall Ratio")
-    return Parameter(
-        selector=selector, value_descriptor=value_descriptor, name=name, **kwargs
-    )
+    return Parameter(selector=selector, value_descriptors=value_descriptor, **kwargs)
 
 
 # Simpler ways to get specific transformers
@@ -615,8 +631,7 @@ def expand_plist(pList: keyFormat) -> List:
     return [
         Parameter(
             FieldSelector(object_name=name, field_name=prop),
-            RangeParameter(min_val=min_, max_val=max_),
-            name=prop,
+            RangeParameter(min_val=min_, max_val=max_, name=prop),
         )
         for name, subProps in pList.items()
         for prop, (min_, max_) in subProps.items()
