@@ -14,10 +14,11 @@ import pandas as pd
 import platypus
 
 # BESOS Imports
-import IO_Objects
-import config
-import objectives
-import parameters
+from besos import IO_Objects
+from besos import config
+from besos import objectives
+from besos import parameters
+from deprecated.sphinx import deprecated
 
 
 # TODO: Consider storing the constraint bounds with the constraints themselves, not in the problem
@@ -67,14 +68,17 @@ class Problem(IO_Objects.ReprMixin):
             If a list is used, strings are converted to Objectives and this list determines the valid inputs.
             If an integer, this problem requires that many constraints.
         :param add_outputs: Outputs that don't need to be optimized in optimization algorithm
-        :param constraint_bounds: A list of strings that are parsable by platypus, and which represent bounds
-            on the corresponding constraints.
-        :param minimize_outputs: A list of booleans indicating if the corresponding output should be
-            minimized or maximized. (Default is to minimize all outputs).
+        :param constraint_bounds: a list of platypus-style constraint bounds, such as "<=750".
+            These are used when converting constraints for use with platypus.
+            Check the platypus documentation for more details.
+        :param minimize_outputs: A list with true/false values corresponding to each output.
+            Outputs having a corresponding value of True will be minimized, while outputs
+            having a corresponding value of False will be maximized instead.
         :param converters: A dictionary with keys from {"outputs", "constraints"} where values indicate how to
             convert those kinds of values to appropriate objectives/constraints for this problem.
         """
         super().__init__()
+        # TODO remove self.converters, use as an argument instead
         self.converters = converters or self.default_converters
         extra_keys = set(self.converters.keys()) - set(self.valid_parts)
         if extra_keys:
@@ -82,28 +86,27 @@ class Problem(IO_Objects.ReprMixin):
                 f"The keys {extra_keys} are not valid for this Problem. Only {self.valid_parts} are valid"
             )
 
-        self.inputs = self._io_to_list(inputs, "inputs")
-        self.num_inputs = len(self.inputs)
-        self.depend_params = {}
+        self.parameters = self._io_to_list(inputs, "inputs")
+        self.value_descriptors = self._get_descriptors(self.parameters)
 
         self.outputs = self._io_to_list(outputs, "outputs")
         self.add_outputs = self._io_to_list(add_outputs, "outputs")
         self.add_outputs_list = None
-        self.num_outputs = len(self.outputs)
+
+        # TODO: Move this information to the output objects
         self.minimize_outputs = minimize_outputs or [True] * self.num_outputs
         msg = "outputs and minimize_outputs must have the same length"
         assert len(self.minimize_outputs) == self.num_outputs, msg
 
         self.constraints = self._io_to_list(constraints, "constraints")
-        self.num_constraints = len(self.constraints)
         # TODO: consider using platypus's constraints here
         self.constraint_bounds = constraint_bounds or []
         msg = "constraints and constraint_bounds must have the same length"
         assert len(self.constraint_bounds) == self.num_constraints, msg
         self.fix_names()
+        self._add_repr("inputs", "parameters", check=True)
         self._add_reprs(
             [
-                "inputs",
                 "outputs",
                 "minimize_outputs",
                 "constraints",
@@ -113,10 +116,21 @@ class Problem(IO_Objects.ReprMixin):
             check=True,
         )
 
+    @staticmethod
+    def _get_descriptors(param_list: List[parameters.Parameter]):
+        found = set()
+        ordered_descriptors = []
+        for parameter in param_list:
+            for descriptor in parameter.value_descriptors:
+                if descriptor not in found:
+                    ordered_descriptors.append(descriptor)
+                    found.add(descriptor)
+        return ordered_descriptors
+
     def fix_names(self):
         mapping = {}
         duplicates = []
-        for obj in self:
+        for obj in self.value_descriptors + self.outputs + self.constraints:
             mapping[obj.name] = mapping.get(obj.name, []) + [obj]
         for name, objects in mapping.items():
             if len(objects) != 1:
@@ -151,13 +165,19 @@ class Problem(IO_Objects.ReprMixin):
             return []
         if isinstance(io_objects, int):
             if part == "inputs":
-                class_ = parameters.Parameter
+                # this repeats the default value for a parameter with no input specified
+                # and should probably be refactored.
+                def constructor(name):
+                    return parameters.Parameter(
+                        value_descriptors=IO_Objects.AnyValue(name=name)
+                    )
+
             elif part in ["outputs", "constraints"]:
-                class_ = IO_Objects.Objective
+                constructor = IO_Objects.Objective
             else:
                 raise ValueError(f"Cannot produce dummy values for part {part}")
-            return [class_(name=f"{part}_{i}") for i in range(io_objects)]
-        if isinstance(io_objects, (str, IO_Objects.IOBase)):
+            return [constructor(name=f"{part}_{i}") for i in range(io_objects)]
+        if isinstance(io_objects, (str, IO_Objects.IOBase, parameters.Parameter)):
             io_objects = [io_objects]
         return [self.convert(o, part) for o in io_objects]
 
@@ -212,6 +232,8 @@ class Problem(IO_Objects.ReprMixin):
         parts = self.expand_parts(parts)
         names = []
         for attr in parts:
+            if attr == "inputs":
+                attr = "value_descriptors"
             if attr == "violation":
                 names.append("violation")
             else:
@@ -272,8 +294,8 @@ class Problem(IO_Objects.ReprMixin):
         problem = platypus.Problem(
             self.num_inputs, self.num_outputs, self.num_constraints
         )
-        for i, parameter in enumerate(self.inputs):
-            problem.types[i] = parameter.platypus_type
+        for i, descriptor in enumerate(self.value_descriptors):
+            problem.types[i] = descriptor.platypus_type
         for i, direction in enumerate(self.minimize_outputs):
             problem.directions[i] = (
                 platypus.Problem.MINIMIZE if direction else platypus.Problem.MAXIMIZE
@@ -283,30 +305,10 @@ class Problem(IO_Objects.ReprMixin):
         return problem
 
     def pre_optimisation(self):
-        """Prepare optimisation for dependent parameter and non objectives."""
-        for i in range(self.num_inputs):
-            if type(self.inputs[i].value_descriptor).__name__ == "DependentParameter":
-                self.depend_params[i] = []
+        """Prepare optimisation for non objectives."""
         if self.add_outputs:
             self.add_outputs_list = []
 
-    def overwrite_values(self, values):
-        """Replace dataframe value with dependent parameter's value."""
-        if not self.depend_params:
-            return values
-        l_values = list(values)
-        for dp in self.depend_params:
-            depend_value = self.get_depend_value(
-                self.inputs[dp].value_descriptor, l_values
-            )
-            l_values[dp] = depend_value
-            self.depend_params[dp].append(depend_value)
-        return tuple(l_values)
-
-    def get_depend_value(self, depend_param, values):
-        """Return value of dependent parameter"""
-        return depend_param.get_value(values[depend_param.index])
-
     def record_results(self, inputs, results):
         """Record add_outputs results"""
         inputs = list(inputs)
@@ -330,11 +332,7 @@ class Problem(IO_Objects.ReprMixin):
         return l
 
     def overwrite_df(self, df):
-        """Insert dependent parameter's value and/or add_outputs' value to dataframe"""
-        labels = df.columns.values
-        for index in self.depend_params:
-            df.drop(columns=labels[index], inplace=True)
-            df.insert(index, labels[index], self.depend_params[index])
+        """Insert add_outputs' values to dataframe"""
         if self.add_outputs:
             l = self.get_non_objective(df)
             for i, data in enumerate(l):
@@ -346,9 +344,20 @@ class Problem(IO_Objects.ReprMixin):
         return df
 
     def post_optimazation(self):
-        self.depend_params = {}
         self.add_outputs_list = None
 
+    @property
+    def num_inputs(self):
+        return len(self.value_descriptors)
+
+    @property
+    def num_outputs(self):
+        return len(self.outputs)
+
+    @property
+    def num_constraints(self):
+        return len(self.constraints)
+
     def __eq__(self, other):
         return (
             self.__class__ is other.__class__
@@ -358,7 +367,15 @@ class Problem(IO_Objects.ReprMixin):
         )
 
     def __iter__(self):
-        return iter(self.inputs + self.outputs + self.constraints)
+        return iter(self.parameters + self.outputs + self.constraints)
+
+    @property
+    @deprecated(
+        version="2.0",
+        reason="Problem.inputs is ambiguous, use .value_descriptors or .parameters instead.",
+    )
+    def inputs(self):
+        return self.parameters
 
 
 # TODO: consider having shortcuts for the converters instead of making this a whole different class
