@@ -11,10 +11,11 @@ from typing import List
 import numpy as np
 import pandas as pd
 import pyehub
+from deprecated import deprecated
 from pyehub.energy_hub.ehub_model import EHubModel
 
 # BESOS Imports
-import config
+from besos import config
 
 
 """This file is a collection of functions to interact with PyEHub."""
@@ -59,6 +60,10 @@ def name_getter(dict):
 
 
 # TODO: Is this really needed? Can we just use the transformation function?
+@deprecated(
+    reason="Function re-implements the evaluator's set phase, "
+    "and cannot handle parameters with multiple inputs."
+)
 def pyehub_parameter_editor(
     hub, parameters: List["parameters.Parameter"], values: list
 ):
