@@ -12,8 +12,8 @@ import pandas as pd
 import pyDOE2
 
 # BESOS Imports
-from IO_Objects import Descriptor
-from problem import Problem
+from besos.IO_Objects import Descriptor
+from besos.problem import Problem
 
 
 ########################################################################################################
@@ -40,16 +40,14 @@ def dist_sampler(
 
     """
     samples = sampler(num_samples, problem.num_inputs, *args, **kwargs)
-    data = {}
-    for i, (name, param) in enumerate(zip(problem.names("inputs"), problem.inputs)):
-        # TODO: is there a better way to do this?
-        if type(param.value_descriptor).__name__ == "DependentParameter":
-            values = data[problem.names("inputs")[param.value_descriptor.index]]
-            data[name] = _sample_param(param, values)
-        else:
-            data[name] = _sample_param(param, samples[:, i])
+    data = {
+        value_descriptor.name: _sample_param(value_descriptor, samples[:, i])
+        for i, (value_descriptor) in enumerate(problem.value_descriptors)
+    }
+
     df = pd.DataFrame(data)
     # enforce the correct order in case it was lost by the dictionary
+    # this may be unnecessary with python 3.7+
     df = df[problem.names("inputs")]
     return df
 
@@ -82,13 +80,13 @@ def extremes(samples: int, attributes: int = None) -> np.ndarray:
     return np.array([np.zeros(attributes), np.ones(attributes)])
 
 
-def _sample_param(parameter: Descriptor, values):
+def _sample_param(descriptor: Descriptor, values):
     try:
         # this will only work if p.sample is numpy compatible
-        return parameter.sample(values)
+        return descriptor.sample(values)
     except TypeError:
         # apply non-vectorised version of p.sample
-        return [parameter.sample(x) for x in values]
+        return [descriptor.sample(x) for x in values]
 
 
 # all of the following return a 2d array of shape (samples, attributes)
