@@ -16,11 +16,7 @@ def test_rbfopt():
     def objective_function(x):
         return (x[0] * x[1] - x[2],)
 
-    p1 = Parameter(value_descriptor=RangeParameter(1, 10), name="None_0")
-    p2 = Parameter(value_descriptor=RangeParameter(0.1, 0.9), name="None_1")
-    p3 = Parameter(value_descriptor=RangeParameter(0, 10), name="None_2")
-    param_list = [p1, p2, p3]
-
+    param_list = [Parameter(value_descriptors=RangeParameter(0, 10)) for _ in range(3)]
     problem = Problem(param_list, 1)
 
     evaluator = EvaluatorGeneric(objective_function, problem, error_mode="Silent")
@@ -28,9 +24,14 @@ def test_rbfopt():
     opt = rbf_opt(evaluator, 30)
     value = opt.iloc[0]
 
+    print(opt)
+
     assert (
-        np.isclose(value["None_0"], 1.0)
-        and np.isclose(value["None_1"], 0.1)
-        and np.isclose(value["None_2"], 10)
-        and np.isclose(value["outputs_0"], -9.9)
+        np.isclose(value["RangeParameter [0, 10]"], 0)
+        and (
+            np.isclose(value["RangeParameter [0, 10]_1"], 5.062882681226724)
+            or np.isclose(value["RangeParameter [0, 10]_1"], 0.5148690)
+        )
+        and np.isclose(value["RangeParameter [0, 10]_2"], 10)
+        and np.isclose(value["outputs_0"], -10)
     ), f"Unexpected output: {value}"  # None_1 assert values hardcoded to pass on Debian
